@@ -1,5 +1,5 @@
 <?php if (!defined('PmWiki')) exit();
-/*  Copyright 2004-2017 Patrick R. Michaud (pmichaud@pobox.com)
+/*  Copyright 2004-2024 Patrick R. Michaud (pmichaud@pobox.com)
     This file is part of PmWiki; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published
     by the Free Software Foundation; either version 2 of the License, or
@@ -24,18 +24,11 @@ global $HTTPHeaders, $KeepToken, $pagename,
   $Charset, $HTMLHeaderFmt, $StrFoldFunction, $AsSpacedFunction;
 
 $Charset = 'UTF-8';
-$HTTPHeaders['utf-8'] = 'Content-type: text/html; charset=UTF-8';
-$HTMLHeaderFmt['utf-8'] = 
+$HTTPHeaders['Content-type'] = 'Content-type: text/html; charset=UTF-8';
+$HTMLHeader1Fmt['Content-type'] = 
   "<meta http-equiv='Content-Type' content='text/html; charset=utf-8' />";
-SDVA($HTMLStylesFmt, array('rtl-ltr' => "
-  .rtl {direction:rtl; unicode-bidi:bidi-override;}
-  .ltr {direction:ltr; unicode-bidi:bidi-override;}
-  .rtl .indent, .rtl.indent, .rtl .outdent, .rtl.outdent {
-    margin-left:0; margin-right: 40px;
-  }
-  "));
-$pagename = @$_REQUEST['n'];
-if (!$pagename) $pagename = @$_REQUEST['pagename'];
+$pagename = strval(@$_REQUEST['n']);
+if (!$pagename) $pagename = strval(@$_REQUEST['pagename']);
 if (!$pagename &&
       preg_match('!^'.preg_quote($_SERVER['SCRIPT_NAME'],'!').'/?([^?]*)!',
           $_SERVER['REQUEST_URI'],$match))
@@ -55,14 +48,15 @@ SDV($MakePageNamePatterns, array(
     '/[?#].*$/' => '',                     # strip everything after ? or #
     "/'/" => '',                           # strip single-quotes
     "/[^$PageNameChars]+/" => ' ',         # convert everything else to space
-    '/(?<=^| )([a-z])/' => PCCF("return strtoupper(\$m[1]);"),
-    '/(?<=^| )([\\xc0-\\xdf].)/' => PCCF("return utf8toupper(\$m[1]);"),
+    '/(?<=^| )([a-z])/' => 'cb_toupper',
+    '/(?<=^| )([\\xc0-\\xdf].)/' => 'utf8toupper',
     '/ /' => ''));
 SDV($StrFoldFunction, 'utf8fold');
 
 $AsSpacedFunction = 'AsSpacedUTF8';
 
 function utf8toupper($x) {
+  if(is_array($x)) $x = $x[1];
   global $CaseConversions;
   if (strlen($x) <= 2 && @$CaseConversions[$x])
     return $CaseConversions[$x];
@@ -74,8 +68,8 @@ function utf8toupper($x) {
   return str_replace($lower, $upper, $x);
 }
 
-
 function utf8fold($x) {
+  if(is_null($x) || $x === '') return '';
   global $StringFolding;
   static $source, $target;
   if (!@$source) {
@@ -109,31 +103,35 @@ SDVA($MarkupExpr, array(
   'toupper' => 'utf8string($args[0], "toupper")',
 ));
 
-function utf8string($str, $start=false, $len=false) { # strlen+substr++ combo for UTF-8
+function utf8string($str=false, $start=false, $len=false) { # strlen+substr++ combo for UTF-8
   global $CaseConversions;
   static $lower;
+  if ($str===false || $str === '') return '';
   if (!@$lower) $lower = implode('|', array_keys($CaseConversions));
   $ascii = preg_match('/[\\x80-\\xFF]/', $str)? 0:1;
   switch ((string)$start) {
     case 'ucfirst': return $ascii ? ucfirst($str) :
-      PPRE("/^($lower)/", '$GLOBALS["CaseConversions"][$m[1]]', $str);
+      preg_replace_callback("/(^)($lower)/", 'cb_uc_first_words', $str);
     case 'ucwords': return $ascii ? ucwords($str) :
-      PPRE("/(^|\\s+)($lower)/", '$m[1].$GLOBALS["CaseConversions"][$m[2]]', $str);
+      preg_replace_callback("/(^|\\s+)($lower)/", 'cb_uc_first_words', $str);
     case 'tolower': return $ascii ? strtolower($str) : utf8fold($str);
     case 'toupper': return $ascii ? strtoupper($str) : utf8toupper($str);
   }
   if ($ascii) {
     if ($start==='strlen') return strlen($str);
-    if ($len===false) return substr($str, $start);
-    return substr($str, $start, $len);
+    if ($len===false) return substr($str, intval($start));
+    return substr($str, intval($start), intval($len));
   }
   $letters = preg_split("/([\\x00-\\x7f]|[\\xc2-\\xdf].|[\\xe0-\\xef]..|[\\xf0-\\xf4]...)/", 
               $str, -1, PREG_SPLIT_DELIM_CAPTURE | PREG_SPLIT_NO_EMPTY);
   if ($start==='strlen') return count($letters);
-  if ($len===false) return implode('', array_slice($letters, $start));
-  return implode('', array_slice($letters, $start, $len));
+  $start = intval($start);
+  if (!is_numeric($len)) return implode('', array_slice($letters, $start));
+  return implode('', array_slice($letters, $start, intval($len)));
+}
+function cb_uc_first_words($m){
+  return $m[1].$GLOBALS["CaseConversions"][$m[2]];
 }
-
 ##   Conversion tables.  
 ##   $CaseConversion maps lowercase utf8 sequences to 
 ##   their uppercase equivalents.  The table was derived from [1].
